@@ -10,6 +10,7 @@ from dipy.core.geometry import vector_norm, vec2vec_rotmat
 from dipy.core.sphere import disperse_charges, HemiSphere
 
 from dipy.utils.deprecator import deprecate_with_version
+from dipy.utils.deprecator import deprecated_params
 
 
 WATER_GYROMAGNETIC_RATIO = 267.513e6  # 1/(sT)
@@ -211,6 +212,37 @@ class GradientTable(object):
         denom = denom.reshape((-1, 1))
         return self.gradients / denom
 
+    def __getitem__(self, idx):
+        if isinstance(idx, int):
+            idx = [idx]  # convert in a list if integer.
+
+        elif isinstance(idx, slice):
+            # Get the lower bound of the slice
+            slice_start = idx.start if idx.start is not None else 0
+            # Check if it is different from b0_threshold
+            if slice_start != self.b0_threshold:
+                # Update b0_threshold and warn the user
+                self.b0_threshold = slice_start
+                warn(f"Updating b0_threshold to {slice_start} for slicing.",
+                     UserWarning, stacklevel=2)
+                idx = range(*idx.indices(len(self.bvals)))
+
+        mask = self.bvals[idx] > self.b0_threshold
+        # Apply the mask to select the desired b-values and b-vectors
+        bvals_selected = self.bvals[idx][mask]
+        bvecs_selected = self.bvecs[idx, :][mask, :]
+
+        # Create a new MyGradientTable object with the selected b-values
+        # and b-vectors
+        return gradient_table_from_bvals_bvecs(bvals_selected,
+                                               bvecs_selected,
+                                               big_delta=self.big_delta,
+                                               small_delta=self.small_delta,
+                                               b0_threshold=self.b0_threshold,
+                                               btens=self.btens)
+        # removing atol parameter as it's not in the constructor
+        # of GradientTable.
+
     @property
     def info(self, use_logging=False):
         show = logging.info if use_logging else print
@@ -859,7 +891,7 @@ def check_multi_b(gtab, n_bvals, non_zero=True, bmag=None):
 
 
 def _btens_to_params_2d(btens_2d, ztol):
-    """Compute trace, anisotropy and assymetry parameters from a single b-tensor
+    """Compute trace, anisotropy and asymmetry parameters from a single b-tensor
 
     Auxiliary function where calculation of `bval`, bdelta` and `b_eta` from a
     (3,3) b-tensor takes place. The main function `btens_to_params` then wraps
@@ -880,7 +912,7 @@ def _btens_to_params_2d(btens_2d, ztol):
     bdelta: float
         normalized tensor anisotropy
     bdelta: float
-        tensor assymetry
+        tensor asymmetry
 
     Notes
     -----
@@ -932,7 +964,7 @@ def _btens_to_params_2d(btens_2d, ztol):
 
 
 def btens_to_params(btens, ztol=1e-10):
-    r"""Compute trace, anisotropy and assymetry parameters from b-tensors.
+    r"""Compute trace, anisotropy and asymmetry parameters from b-tensors.
 
     Parameters
     ----------
@@ -948,7 +980,7 @@ def btens_to_params(btens, ztol=1e-10):
     bdelta: numpy.ndarray
         normalized tensor anisotropy(s)
     b_eta: numpy.ndarray
-        tensor assymetry(s)
+        tensor asymmetry(s)
 
     Notes
     -----
@@ -1003,7 +1035,7 @@ def btens_to_params(btens, ztol=1e-10):
 
 
 def params_to_btens(bval, bdelta, b_eta):
-    """Compute b-tensor from trace, anisotropy and assymetry parameters.
+    """Compute b-tensor from trace, anisotropy and asymmetry parameters.
 
     Parameters
     ----------
@@ -1012,7 +1044,7 @@ def params_to_btens(bval, bdelta, b_eta):
     bdelta: int or float
         normalized tensor anisotropy (>= -0.5 and <= 1)
     b_eta: int or float
-        tensor assymetry (>= 0 and <= 1)
+        tensor asymmetry (>= 0 and <= 1)
 
     Returns
     -------
