@@ -15,6 +15,7 @@ from dipy.data.fetcher import (get_fnames,
                                read_scil_b0,
                                fetch_stanford_hardi,
                                read_stanford_hardi,
+                               fetch_stanford_tracks,
                                fetch_taiwan_ntu_dsi,
                                read_taiwan_ntu_dsi,
                                fetch_sherbrooke_3shell,
@@ -50,11 +51,17 @@ from dipy.data.fetcher import (get_fnames,
                                fetch_bundle_fa_hcp,
                                fetch_gold_standard_io,
                                fetch_resdnn_weights,
+                               fetch_synb0_weights,
+                               fetch_synb0_test,
+                               fetch_evac_weights,
+                               fetch_evac_test,
                                read_qte_lte_pte,
                                read_DiB_70_lte_pte_ste,
                                read_DiB_217_lte_pte_ste,
                                read_five_af_bundles,
-                               fetch_hbn)
+                               fetch_hbn,
+                               fetch_ptt_minimal_dataset,
+                               fetch_bundle_warp_dataset)
 
 from ..utils.arrfuncs import as_native_array
 from dipy.io.image import load_nifti
