@@ -19,10 +19,13 @@ from dipy.core.gradients import (gradient_table,
                                  gradient_table_from_gradient_strength_bvecs)
 from dipy.io.gradients import read_bvals_bvecs
 from dipy.io.image import load_nifti, load_nifti_data, save_nifti
+
 from dipy.io.streamline import load_trk
+
 from dipy.utils.optpkg import optional_package, TripWire
 
 
+
 from urllib.request import urlopen
 
 # Set a user-writeable file-system location to put files:
@@ -137,6 +140,7 @@ def fetch_data(files, folder, data_size=None):
         value. The downloaded file is not deleted when this error is raised.
 
     """
+
     if not op.exists(folder):
         _log("Creating new folder %s" % folder)
         os.makedirs(folder)
@@ -288,6 +292,52 @@ fetch_resdnn_weights = _make_fetcher(
     ['f0e118d72ab804a464494bd9015227f4'],
     doc="Download ResDNN model weights for Nath et. al 2018")
 
+fetch_synb0_weights = _make_fetcher(
+    "fetch_synb0_weights",
+    pjoin(dipy_home, 'synb0'),
+    'https://ndownloader.figshare.com/files/',
+    ['36379914', '36379917', '36379920', '36379923', '36379926'],
+    ['synb0_default_weights1.h5',
+     'synb0_default_weights2.h5',
+     'synb0_default_weights3.h5',
+     'synb0_default_weights4.h5',
+     'synb0_default_weights5.h5'],
+    ['a9362c75bc28616167a11a42fe5d004e',
+     '9dc9353d6ff741d8e22b8569f157c56e',
+     'e548f341e4f12d63dfbed306233fddce',
+     '8cb7a3492d08e4c9b8938277d6fd9b75',
+     '5e796f892605b3bdb9cb9678f1c6ac11'],
+    doc="Download Synb0 model weights for Schilling et. al 2019")
+
+fetch_synb0_test = _make_fetcher(
+    "fetch_synb0_test",
+    pjoin(dipy_home, 'synb0'),
+    'https://ndownloader.figshare.com/files/',
+    ['36379911', '36671850'],
+    ['test_input_synb0.npz',
+     'test_output_synb0.npz'],
+    ['987203aa73de2dac8770f39ed506dc0c',
+     '515544fbcafd9769785502821b47b661'],
+    doc="Download Synb0 test data for Schilling et. al 2019")
+
+fetch_evac_weights = _make_fetcher(
+    "fetch_evac_weights",
+    pjoin(dipy_home, 'evac'),
+    'https://ndownloader.figshare.com/files/',
+    ['40150867'],
+    ['evac_default_weights.h5'],
+    ['998d5122e0aef1ccf7c0d58e41e978af'],
+    doc="Download EVAC+ model weights for Park et. al 2022")
+
+fetch_evac_test = _make_fetcher(
+    "fetch_evac_test",
+    pjoin(dipy_home, 'evac'),
+    'https://ndownloader.figshare.com/files/',
+    ['40150894'],
+    ['evac_test_data.npz'],
+    ['a2173e8f800ab7ab3b159b86bf3a8536'],
+    doc="Download EVAC+ test data for Park et. al 2022")
+
 fetch_stanford_t1 = _make_fetcher(
     "fetch_stanford_t1",
     pjoin(dipy_home, 'stanford_hardi'),
@@ -306,6 +356,16 @@ fetch_stanford_pve_maps = _make_fetcher(
      '1654b20aeb35fc2734a0d7928b713874',
      '2e244983cf92aaf9f9d37bc7716b37d5'])
 
+fetch_stanford_tracks = _make_fetcher(
+    "fetch_stanford_tracks",
+    pjoin(dipy_home, 'stanford_hardi'),
+    'https://raw.githubusercontent.com/dipy/dipy_datatest/main/',
+    ['hardi-lr-superiorfrontal.trk', ],
+    ['hardi-lr-superiorfrontal.trk', ],
+    ['2d49aaf6ad6c10d8d069bfb319bf3541',],
+    doc="Download stanford track for examples",
+    data_size="1.4MB")
+
 fetch_taiwan_ntu_dsi = _make_fetcher(
     "fetch_taiwan_ntu_dsi",
     pjoin(dipy_home, 'taiwan_ntu_dsi'),
@@ -590,6 +650,30 @@ fetch_DiB_217_lte_pte_ste = _make_fetcher(
     data_size='166.3 MB')
 
 
+fetch_ptt_minimal_dataset = _make_fetcher(
+    "fetch_ptt_minimal_dataset",
+    pjoin(dipy_home, 'ptt_dataset'),
+    'https://raw.githubusercontent.com/dipy/dipy_datatest/main/',
+    ['ptt_fod.nii', 'ptt_seed_coords.txt', 'ptt_seed_image.nii'],
+    ['ptt_fod.nii', 'ptt_seed_coords.txt', 'ptt_seed_image.nii'],
+    ['6e454f8088b64e7b85218c71010d8dbe',
+     '8c2d71fb95020e2bb1743623eb11c2a6',
+     '9cb88f88d664019ba80c0b372c8bafec'],
+    doc="Download FOD and seeds for PTT testing and examples",
+    data_size="203KB")
+
+
+fetch_bundle_warp_dataset = _make_fetcher(
+    "fetch_bundle_warp_dataset",
+    pjoin(dipy_home, 'bundle_warp'),
+    'https://ndownloader.figshare.com/files/',
+    ['40026343', '40026346'],
+    ['m_UF_L.trk', 's_UF_L.trk', ],
+    ['4db38ca1e80c16d6e3a97f88f0611187',
+     'c1499005baccfab865ce38368d7a4c7f'],
+    doc="Download Bundle Warp dataset")
+
+
 def get_fnames(name='small_64D'):
     """Provide full paths to example or test datasets.
 
@@ -779,6 +863,27 @@ def get_fnames(name='small_64D'):
         files, folder = fetch_resdnn_weights()
         wraw = pjoin(folder, 'resdnn_weights_mri_2018.h5')
         return wraw
+    if name == 'synb0_default_weights':
+        _, folder = fetch_synb0_weights()
+        w1 = pjoin(folder, 'synb0_default_weights1.h5')
+        w2 = pjoin(folder, 'synb0_default_weights2.h5')
+        w3 = pjoin(folder, 'synb0_default_weights3.h5')
+        w4 = pjoin(folder, 'synb0_default_weights4.h5')
+        w5 = pjoin(folder, 'synb0_default_weights5.h5')
+        return w1, w2, w3, w4, w5
+    if name == 'synb0_test_data':
+        files, folder = fetch_synb0_test()
+        input_array = pjoin(folder, 'test_input_synb0.npz')
+        target_array = pjoin(folder, 'test_output_synb0.npz')
+        return input_array, target_array
+    if name == 'evac_default_weights':
+        files, folder = fetch_evac_weights()
+        weight = pjoin(folder, 'evac_default_weights.h5')
+        return weight
+    if name == 'evac_test_data':
+        files, folder = fetch_evac_test()
+        test_data = pjoin(folder, 'evac_test_data.npz')
+        return test_data
     if name == 'DiB_70_lte_pte_ste':
         _, folder = fetch_DiB_70_lte_pte_ste()
         fdata = pjoin(folder, 'DiB_70_lte_pte_ste.nii.gz')
@@ -794,6 +899,12 @@ def get_fnames(name='small_64D'):
         fbvec = pjoin(folder, 'bvec_DiB_217_lte_pte_ste.bvec')
         fmask = pjoin(folder, 'DiB_mask.nii.gz')
         return fdata_1, fdata_2, fbval, fbvec, fmask
+    if name == 'ptt_minimal_dataset':
+        files, folder = fetch_ptt_minimal_dataset()
+        fod_name = pjoin(folder, 'ptt_fod.nii')
+        seed_coords_name = pjoin(folder, 'ptt_seed_coords.txt')
+        seed_image_name = pjoin(folder, 'ptt_seed_image.nii')
+        return fod_name, seed_coords_name, seed_image_name
 
 
 def read_qtdMRI_test_retest_2subjects():
@@ -1055,7 +1166,7 @@ def read_tissue_data(contrast='T1'):
 
     Parameters
     ----------
-    constrast : str
+    contrast : str
         'T1', 'T1 denoised' or 'Anisotropic Power'
 
     Returns
@@ -1162,7 +1273,7 @@ def read_mni_template(version="a", contrast="T2"):
     if contrast == "mask" and version == "a":
         raise ValueError("No template mask available for MNI 2009a")
 
-    if not(isinstance(contrast, str)) and version == "c":
+    if not isinstance(contrast, str) and version == "c":
         for k in contrast:
             if k == "T2":
                 raise ValueError("No T2 image for MNI template 2009c")
@@ -1755,7 +1866,7 @@ def fetch_hcp(subjects,
         data_files[pjoin(sub_dir, 'anat', f'sub-{subject}_T1w.nii.gz')] =\
             f'{study}/{subject}/T1w/T1w_acpc_dc.nii.gz'
         data_files[pjoin(sub_dir, 'anat',
-                           f'sub-{subject}_aparc+aseg_seg.nii.gz')] =\
+                         f'sub-{subject}_aparc+aseg_seg.nii.gz')] =\
             f'{study}/{subject}/T1w/aparc+aseg.nii.gz'
 
     download_files = {}
@@ -1860,7 +1971,8 @@ def fetch_hbn(subjects, path=None):
             Prefix=f"data/Projects/HBN/BIDS_curated/derivatives/qsiprep/sub-{subject}/{ses}/")  # noqa
         query_content = query.get('Contents', None)
         if query_content is None:
-            raise ValueError(f"Could not find derivatives data for subject {subject}")
+            raise ValueError(
+                f"Could not find derivatives data for subject {subject}")
         file_list = [kk["Key"] for kk in query["Contents"]]
         sub_dir = op.join(base_dir, f'sub-{subject}')
         ses_dir = op.join(sub_dir, ses)
@@ -1868,7 +1980,8 @@ def fetch_hbn(subjects, path=None):
             os.makedirs(os.path.join(ses_dir, 'dwi'), exist_ok=True)
             os.makedirs(os.path.join(ses_dir, 'anat'), exist_ok=True)
         for remote in file_list:
-            full = remote.split("Projects")[-1][1:].replace("/BIDS_curated", "")
+            full = remote.split(
+                "Projects")[-1][1:].replace("/BIDS_curated", "")
             local = op.join(dipy_home, full)
             data_files[local] = remote
 
