@@ -6,6 +6,7 @@ Class and helper functions for fitting the Histological ResDNN model.
 
 from packaging.version import Version
 import logging
+import numpy as np
 
 from dipy.core.gradients import unique_bvals_magnitude, get_bval_indices
 from dipy.core.sphere import HemiSphere
@@ -13,7 +14,7 @@ from dipy.data import get_sphere, get_fnames
 from dipy.reconst.shm import sf_to_sh, sh_to_sf, sph_harm_ind_list
 from dipy.testing.decorators import doctest_skip_parser
 from dipy.utils.optpkg import optional_package
-import numpy as np
+from dipy.nn.utils import set_logger_level
 
 tf, have_tf, _ = optional_package('tensorflow')
 if have_tf:
@@ -21,24 +22,17 @@ if have_tf:
     from tensorflow.keras.layers import Input, Dense, Add
     if Version(tf.__version__) < Version('2.0.0'):
         raise ImportError('Please upgrade to TensorFlow 2+')
+else:
+    logging.warning('This model requires Tensorflow.\
+                    Please install these packages using \
+                    pip. If using mac, please refer to this \
+                    link for installation. \
+                    https://github.com/apple/tensorflow_macos')
 
 
 logging.basicConfig()
 logger = logging.getLogger('histo_resdnn')
 
-
-def set_logger_level(log_level):
-    """ Change the logger of the HistoResDNN to one on the following:
-    DEBUG, INFO, WARNING, CRITICAL, ERROR
-
-    Parameters
-    ----------
-    log_level : str
-        Log level for the HistoResDNN only
-    """
-    logger.setLevel(level=log_level)
-
-
 class HistoResDNN:
     """
     This class is intended for the ResDNN Histology Network model.
@@ -94,7 +88,7 @@ class HistoResDNN:
         self.basis_type = basis_type
 
         log_level = 'INFO' if verbose else 'CRITICAL'
-        set_logger_level(log_level)
+        set_logger_level(log_level, logger)
         if self.basis_type != 'tournier07':
             logger.warning('Be careful, original weights were obtained '
                            'from training on the tournier07 basis, '
@@ -169,7 +163,7 @@ class HistoResDNN:
         return self.model.predict(x_test)
 
     def predict(self, data, gtab, mask=None, chunk_size=1000):
-        """ Wrapper function to faciliate prediction of larger dataset.
+        """ Wrapper function to facilitate prediction of larger dataset.
         The function will mask, normalize, split, predict and 're-assemble'
         the data as a volume.
 
