@@ -829,7 +829,7 @@ class TensorModel(ReconstModel):
             dti_params[mask, :] = params_in_mask
             if self.return_S0_hat:
                 S0_params = np.zeros(data.shape[:-1])
-                S0_params[mask] = model_S0
+                S0_params[mask] = model_S0.squeeze()
 
         return TensorFit(self, dti_params, model_S0=S0_params)
 
