@@ -11,7 +11,8 @@ from dipy.direction import (BootDirectionGetter,
                             ClosestPeakDirectionGetter,
                             DeterministicMaximumDirectionGetter,
                             PeaksAndMetrics,
-                            ProbabilisticDirectionGetter)
+                            ProbabilisticDirectionGetter,
+                            PTTDirectionGetter)
 from dipy.reconst.csdeconv import ConstrainedSphericalDeconvModel
 from dipy.reconst.shm import descoteaux07_legacy_msg
 from dipy.tracking.local_tracking import (LocalTracking,
@@ -145,7 +146,7 @@ def test_stop_conditions():
     npt.assert_equal(len(sl), 2)
 
     # The streamline should contain only one point, the seed point,
-    # because no valid inital direction was returned.
+    # because no valid initial direction was returned.
     y = 6
     sl = next(streamlines_all)
     npt.assert_equal(sl[0], seeds[y])
@@ -377,6 +378,15 @@ def test_particle_filtering_tractography():
     npt.assert_(np.array([len(pft_streamlines) > 0]))
     npt.assert_(np.array([len(pft_streamlines) >= len(local_streamlines)]))
 
+    # Test PFT with a PTT direction getter
+    dg_ptt = PTTDirectionGetter.from_pmf(pmf, 60, sphere)
+    pft_ptt_streamlines = Streamlines(ParticleFilteringTracking(
+        dg_ptt, sc, seeds, np.eye(4), step_size, max_cross=1, return_all=False,
+        pft_back_tracking_dist=1, pft_front_tracking_dist=0.5))
+
+    npt.assert_(np.array([len(pft_ptt_streamlines) > 0]))
+    npt.assert_(np.array([len(pft_ptt_streamlines) >= len(local_streamlines)]))
+
     # Test that all points are equally spaced
     for l in [1, 2, 5, 10, 100]:
         pft_streamlines = ParticleFilteringTracking(dg, sc, seeds, np.eye(4),
