@@ -3,7 +3,8 @@ import logging
 from warnings import warn
 import numpy as np
 import nibabel as nib
-
+from os.path import join as pjoin
+from dipy.utils.optpkg import optional_package
 from dipy.align.imaffine import AffineMap
 from dipy.align.imwarp import (SymmetricDiffeomorphicRegistration,
                                DiffeomorphicMap)
@@ -11,12 +12,19 @@ from dipy.align.metrics import CCMetric, SSDMetric, EMMetric
 from dipy.align.reslice import reslice
 from dipy.align import affine_registration, motion_correction
 from dipy.align.streamlinear import slr_with_qbx
+from dipy.tracking.streamline import set_number_of_points
+from dipy.align.streamwarp import bundlewarp
 from dipy.core.gradients import gradient_table
 from dipy.io.image import save_nifti, load_nifti, save_qa_metric
 from dipy.io.gradients import read_bvals_bvecs
 from dipy.tracking.streamline import transform_streamlines
 from dipy.workflows.workflow import Workflow
 
+pd, have_pd, _ = optional_package("pandas")
+
+if have_pd:
+    import pandas as pd
+
 
 def check_dimensions(static, moving):
     """Check the dimensions of the input images.
@@ -701,10 +709,14 @@ class SynRegistrationFlow(Workflow):
 
             mapping_data = np.array([mapping.forward.T, mapping.backward.T]).T
             warped_moving = mapping.transform(moving_image)
+            mapping.is_inverse = True
+            inv_static = mapping.transform(static_image)
 
             # Saving
             logging.info('Saving warped {0}'.format(owarped_file))
             save_nifti(owarped_file, warped_moving, static_grid2world)
+            logging.info('Saving inverse transformes static {0}'.format(out_inv_static))
+            save_nifti(out_inv_static, inv_static, static_grid2world)
             logging.info('Saving Diffeomorphic map {0}'.format(omap_file))
             save_nifti(omap_file, mapping_data, mapping.codomain_world2grid)
 
@@ -772,3 +784,112 @@ class MotionCorrectionFlow(Workflow):
                 for affine_slice in reg_affines:
                     np.savetxt(outfile, affine_slice, fmt='%-7.2f')
                     outfile.write('# New slice\n')
+
+
+class BundleWarpFlow(Workflow):
+
+    @classmethod
+    def get_short_name(cls):
+        return 'bundlewarp'
+
+    def run(self, static_file, moving_file, dist=None, alpha=0.3, beta=20,
+            max_iter=15, affine=True,
+            out_dir='',
+            out_linear_moved='linearly_moved.trk',
+            out_nonlinear_moved='nonlinearly_moved.trk',
+            out_warp_transform='warp_transform.npy',
+            out_warp_kernel='warp_kernel.npy',
+            out_dist='distance_matrix.npy',
+            out_matched_pairs='matched_pairs.npy'):
+        """ BundleWarp: streamline-based nonlinear registration.
+
+        BundleWarp is nonrigid registration method for deformable registration
+        of white matter tracts.
+
+        Parameters
+        ----------
+        static_file : string
+            Path to the static (reference) .trk file.
+        moving_file : string
+            Path to the moving (target to be registered) .trk file.
+        dist : string, optional
+            Path to the precalculated distance matrix file.
+        alpha : float, optional
+            Represents the trade-off between regularizing the deformation and
+            having points match very closely. Lower value of alpha means high
+            deformations. It is represented with λ in BundleWarp paper. NOTE:
+            setting alpha<=0.01 will result in highly deformable registration
+            that could extremely modify the original anatomy of the moving
+            bundle. (default 0.3)
+        beta : int, optional
+            Represents the strength of the interaction between points
+            Gaussian kernel size. (default 20)
+        max_iter : int, optional
+            Maximum number of iterations for deformation process in ml-CPD
+            method. (default 15)
+        affine : boolean, optional
+            If False, use rigid registration as starting point. (default True)
+        out_dir : string, optional
+            Output directory. (default current directory)
+        out_linear_moved : string, optional
+            Filename of linearly moved bundle.
+        out_nonlinear_moved : string, optional
+            Filename of nonlinearly moved (warped) bundle.
+        out_warp_transform : string, optional
+            Filename of warp transformations generated by BundleWarp.
+        out_warp_kernel : string, optional
+            Filename of regularization gaussian kernel generated by BundleWarp.
+        out_dist : string, optional
+            Filename of MDF distance matrix.
+        out_matched_pairs : string, optional
+            Filename of matched pairs; treamline correspondences between two
+            bundles.
+
+        References
+        ----------
+         .. [Chandio2023] Chandio et al. "BundleWarp, streamline-based nonlinear
+            registration of white matter tracts." bioRxiv (2023): 2023-01.
+        """
+
+        logging.info('Loading static file {0}'.format(static_file))
+        logging.info('Loading moving file {0}'.format(moving_file))
+
+        static_obj = nib.streamlines.load(static_file)
+        moving_obj = nib.streamlines.load(moving_file)
+
+        static, static_header = static_obj.streamlines, static_obj.header
+        moving, moving_header = moving_obj.streamlines, moving_obj.header
+
+        static = set_number_of_points(static, 20)
+        moving = set_number_of_points(moving, 20)
+
+        deformed_bundle, affine_bundle, dists, mp, warp = \
+            bundlewarp(static, moving, dist=dist, alpha=alpha, beta=beta,
+                       max_iter=max_iter, affine=affine)
+
+        logging.info('Saving output file {0}'.format(out_linear_moved))
+        new_tractogram = nib.streamlines.Tractogram(affine_bundle,
+                                                    affine_to_rasmm=np.eye(4))
+        nib.streamlines.save(new_tractogram, pjoin(out_dir, out_linear_moved),
+                             header=moving_header)
+
+        logging.info('Saving output file {0}'.format(out_nonlinear_moved))
+        new_tractogram = nib.streamlines.Tractogram(deformed_bundle,
+                                                    affine_to_rasmm=np.eye(4))
+        nib.streamlines.save(new_tractogram, pjoin(out_dir, out_nonlinear_moved),
+                             header=moving_header)
+
+        df = pd.DataFrame(warp, columns=['transforms', 'gaussian_kernel'])
+
+        logging.info('Saving output file {0}'.format(out_warp_transform))
+        np.save(pjoin(out_dir, out_warp_transform), np.array(df['transforms']))
+
+        logging.info('Saving output file {0}'.format(out_warp_kernel))
+        np.save(pjoin(out_dir, out_warp_kernel),
+                np.array(df['gaussian_kernel']))
+
+        logging.info('Saving output file {0}'.format(out_dist))
+        np.save(pjoin(out_dir, out_dist), dist)
+
+        logging.info('Saving output file {0}'.format(out_matched_pairs))
+        np.save(pjoin(out_dir, out_matched_pairs), mp)
