@@ -7,7 +7,7 @@ set -ex
 
 PIPI="pip install --timeout=60"
 
-if [ "$USE_PRE" == "1" ] || [ "$USE_PRE" = true ]; then
+if [ "$USE_PRE" == "1" ] || [ "$USE_PRE" == true ]; then
     PIPI="$PIPI --extra-index-url=$PRE_WHEELS --pre";
 fi
 
