@@ -14,16 +14,15 @@ cd for_testing
 cp ../setup.cfg .
 # No figure windows for mpl; quote to hide : from travis-ci yaml parsing
 echo "backend : agg" > matplotlibrc
-if [ "$COVERAGE" == "1" ] || [ "$COVERAGE" = true ]; then
+if [ "$COVERAGE" == "1" ] || [ "$COVERAGE" == true ]; then
     cp ../.coveragerc .;
     cp ../.codecov.yml .;
     # Run the tests and check for test coverage.
     coverage run -m pytest -svv --doctest-modules --verbose --durations=10 --pyargs dipy
     coverage report -m  # Generate test coverage report.
-    echo "START UPLOAD COVERAGE"
-    codecov    # Upload the report to codecov.
+    coverage xml  # Generate coverage report in xml format for codecov upload.
 else
     pytest -svv --doctest-modules --verbose --durations=10 --pyargs dipy
 fi
-
+cd ..
 set +ex
\ No newline at end of file
